package mygroup.myartifact;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import mygroup.myartifact.dispatchers.DispatcherAction;
import mygroup.myartifact.dispatchers.Handler;
import mygroup.myartifact.modules.Module;
import mygroup.myartifact.modules.ModuleManager;
import mygroup.myartifact.modules.ModuleManagerListener;
import mygroup.myartifact.modules.Node;
import mygroup.myartifact.modules.NodeListener;
import net.miginfocom.swing.MigLayout;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MachineNodeGUI extends JFrame {
	private Logger _logger;
	private MachineNode _machineNode;
	
	private JPanel contentPane;
	private JTextField idTextField;
	private JButton btnStart;
	private JButton btnStop;
	private List<JCheckBox> dispatcherCheckBoxes;
	private List<JCheckBox> modulesCheckBoxes;

	private NodeTableModel _model;

	/**
	 * Main constructor to start everything, used for normal application start. This constructor *needs* to be executed in Event Dispatcher Thread!!!
	 */
	public MachineNodeGUI(final MachineNode machineNodeParam) {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"); // Little problem: only works with Sun/Oracle JVM
		} catch (Exception e) {
			e.printStackTrace();
		}
		setTitle("Hekate");
		this._logger = LoggerFactory.getLogger(MachineNodeGUI.class);
		this._machineNode = machineNodeParam; // Use MachineNode object to get init configuration, e.g. id, nick, and available capabilities
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE); // Let's run and WindowListener and seems to be very soft
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				_logger.debug("Closing MachineNodeGUI and MachineNode");
				// Of course in separate thread not to block AWT-Thread, but this isn't important, because the application is shutting down anyway...
				_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() {
					@Override
					public void run() {
						_machineNode.getModuleManager().stop();
					}
				});
				_machineNode.getModuleManager().getExecutorSerivce().shutdown();
				// TODO How to tell AWT-Thread to shutdown?

			}
		});

		setSize(520, 450);
		setLocationRelativeTo(null); // Show windows in center of Desktop; TODO remember last position for next start-up?
		contentPane = new JPanel();
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		contentPane.setLayout(new MigLayout(""));

		// General node information
		JLabel generalSeparatorLabel = new JLabel("General information");
		generalSeparatorLabel.setForeground(Color.BLUE);
		contentPane.add(generalSeparatorLabel, "span, gapbottom 1, split 2"); // No wait, span already takes *all* cell's ini this row I guess
		JSeparator generalSeparator = new JSeparator();
		contentPane.add(generalSeparator, "w 100%, wrap");
		JLabel idLabel = new JLabel("ID");
		contentPane.add(idLabel, "split 4"); // contentPane.add(idLabel, "cell 0 0,alignx trailing");
		idTextField = new JTextField();
		idTextField.setText(machineNodeParam.getId());
		contentPane.add(idTextField, "");
		idTextField.setColumns(26);
		JLabel nickLabel = new JLabel("Nickname");
		contentPane.add(nickLabel, "gap 10");
		final JTextField nickTextField = new JTextField();
		nickTextField.setText(machineNodeParam.getNick());
		nickTextField.setColumns(26);
		nickTextField.addKeyListener(new KeyAdapter() {
			private String oldNick = machineNodeParam.getNick();
			@Override
			public void keyReleased(KeyEvent e) {
				int code = e.getKeyCode();
				if (code == KeyEvent.VK_ENTER) {
					if (!oldNick.equals(nickTextField.getText())) { // User inputed new nick after pressing enter, so broadcast new nickname
						// Broadcast new nickname
						_machineNode.setNick(nickTextField.getText());
						oldNick = nickTextField.getText(); // Remember new nickname as old nickname
					}
				}
			}
		});
		contentPane.add(nickTextField, "");

		// Handler
		JLabel dispatchersLabel = new JLabel("Dispatcher");
		dispatchersLabel.setForeground(Color.BLUE);
		contentPane.add(dispatchersLabel, "span, split 2, gapbottom 1, newline 10");
		JSeparator dispatcherSeparator = new JSeparator();
		contentPane.add(dispatcherSeparator, "w 100%, wrap");

		// read current available dispatcher and show them here TODO introduce hook-up mechanism to also load dispatcher while node still running !?!
		dispatcherCheckBoxes = new ArrayList<JCheckBox>();
		// Print 'No dispatchers loaded' if no dispatchers available
		if (machineNodeParam.getDispatchers().size() == 0) {
			JLabel noDispatchersLabel = new JLabel("No dispatchers available.");
			noDispatchersLabel.setForeground(Color.GRAY);
			contentPane.add(noDispatchersLabel, "");
		} else {
			// TODO introduce JLabel wrapper to be able to dynamically add new dispatchers; is MigLayout able to cope with dynamic changed?
 			// maybe this helps: http://stackoverflow.com/questions/4247465/is-there-a-way-to-insert-a-cell-in-miglayout
			// JCheckBox tmpCheckBox = null;
			String split = "split " + machineNodeParam.getDispatchers().size();
			for (final Handler dispatcher : machineNodeParam.getDispatchers()) {
				final JCheckBox tmpCheckBox = new JCheckBox(dispatcher.getIdentifier()); // Use short description identifier instead of full package name
				tmpCheckBox.setSelected(true); // Every loaded Dispatcher at startup is enabled as default, so checking of Dispatcher.isEnabled() obsolete

				tmpCheckBox.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent actionEvent) {
						if (tmpCheckBox.isSelected()) { // Just selected; status changed from unchecked to checked
							// Activate Dispatcher
							dispatcher.setEnabled(true);
						} else {
							// Deactivate Dispatcher
							dispatcher.setEnabled(false);
						}
						_machineNode.enabledDispatchersChanged();
					}
				});
				contentPane.add(tmpCheckBox, split); // I hope this works // contentPane.add(tmpCheckBox, split + machineNodeParam.getDispatchers().size()); // I hope this works
				dispatcherCheckBoxes.add(tmpCheckBox);
				split = "";
			} // Last element needs to be wrapped, how to do this ad hoc? Maybe introduce not visible, 0x0 pixel element with wrap to solve this problem; or let next line wrap automatically => use 'newline' instead of 'wrap'
		}

		// Modules
		JLabel modulesLabel = new JLabel("Modules");
		modulesLabel.setForeground(Color.BLUE);
		contentPane.add(modulesLabel, "span, gapbottom 1, split 2, newline 10");
		JSeparator modulesSeparator = new JSeparator();
		contentPane.add(modulesSeparator, "w 100%, wrap");
		modulesCheckBoxes = new ArrayList<JCheckBox>();
		if (machineNodeParam.getModuleManager().getModules().size() == 0) {
			JLabel noModulesLabel = new JLabel("No modules available.");
			noModulesLabel.setForeground(Color.GRAY);
			contentPane.add(noModulesLabel, "");
		} else {
			JCheckBox tmpCheckBox = null;
			String split = "split " + _machineNode.getModuleManager().getModules().size();
			for (Module module : _machineNode.getModuleManager().getModules()) {
				tmpCheckBox = new JCheckBox(module.getIdentifier());
				tmpCheckBox.setSelected(true);
				contentPane.add(tmpCheckBox, split);
				modulesCheckBoxes.add(tmpCheckBox);
				split = "";
			}
		}

		// Control buttons
		JLabel controlsLabel = new JLabel("Controls");
		controlsLabel.setForeground(Color.BLUE);
		contentPane.add(controlsLabel, "span, gapbottom 1, split 2, newline 10"); // contentPane.add(dispatcherSeparatorLabel, "span, gapbottom 1, split 2");
		JSeparator controlsSeparator = new JSeparator();
		contentPane.add(controlsSeparator, "w 100%, wrap");

		btnStart = new JButton("Start");
		btnStart.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { // User pressed on Start button, so start the discovery and awareness process; no need to check left/right mouse-click with actionPeformed
				_logger.debug("Pressed on START-button, booting up MachineNode ...");
				btnStart.setEnabled(false);
				btnStop.setEnabled(true);
				idTextField.setEditable(false); // Can't change id after node started; nick can be still changed
				
				String nick = nickTextField.getText(); // Read nick from GUI-Field set MachineNode.nickname with it, because user could enter new nick *without* pressing ENTER
				_machineNode.setNick(nick);				
				
				_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() { // Let's decouple from AWT-Thread
					@Override
					public void run() {
						_machineNode.getModuleManager().start();
					};
				});
			}
		});
		contentPane.add(btnStart, "split 2"); // contentPane.add(btnStart, "newline 10"); // contentPane.add(btnStart, "cell 6 1,growx");

		btnStop = new JButton("Stop");
		btnStop.setEnabled(false);
		btnStop.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) { // User pressed on Stop button, so stop discovery and awareness process
				_logger.debug("Pressed on STOP-button, shutting down MachineNode ...");
				btnStart.setEnabled(true);
				btnStop.setEnabled(false);
				idTextField.setEditable(true); // User can change node ID, because node is not part of known node network anymore

				_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() {
					@Override
					public void run() {
						_machineNode.stop();
					};
				});
			}
		});
		contentPane.add(btnStop, ""); // contentPane.add(btnStop, "cell 6 2,growx");

		// Found nodes; Area for showing found nodes; using table seems to be OK, but how show available dispatcher, which could be a lot? Simple column OK for now
		JLabel nodesLabel = new JLabel("Found nodes");
		controlsLabel.setForeground(Color.BLUE);
		contentPane.add(nodesLabel, "span, gapbottom 1, split 2, newline 10"); // contentPane.add(dispatcherSeparatorLabel, "span, gapbottom 1, split 2");
		JSeparator nodesSeparator = new JSeparator();
		contentPane.add(nodesSeparator, "w 100%, wrap");
		
		createTableGUI();
	}

	private void createTableGUI() {
		_model = new NodeTableModel();
		final JTable table = new JTable(_model);
		_machineNode.getModuleManager().addListener(this._model); // Get notified when new nodes are available

		// Custom cell renderer for first column to draw red/green circle for true/false Boolean
//		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		TableColumnModel columnTable = table.getColumnModel();
//		TableColumn statusColumn = columnTable.getColumn(1);
//		statusColumn.setPreferredWidth(210);
//		statusColumn = columnTable.getColumn(2);
//		statusColumn.setPreferredWidth(210);
		TableColumn statusColumn = columnTable.getColumn(0);
		statusColumn.setResizable(false);
		statusColumn.setPreferredWidth(25); // TODO Shrink width of first column, red/green icon doesn't need that much
		statusColumn.setMaxWidth(25);
		statusColumn.setMinWidth(25);
		statusColumn.setCellRenderer(new TableCellRenderer() {
					@Override
					public Component getTableCellRendererComponent(final JTable tableParam, final Object value, boolean isSelected, boolean hasFocus, int row, int column) {
						JPanel panel = new JPanel() { // Unfortunately reusing same JPanel is not possible here, because width+height of cell could change on every call; I hope JTable is smart enough to reuse the JPanel
							@Override
							public void paintComponent(Graphics g2) {
								int width = tableParam.getColumnModel().getColumn(0).getWidth();
								int height = tableParam.getRowHeight();
								Graphics2D g = (Graphics2D) g2;
								g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

								// Value should always be a boolean
								if (value instanceof Boolean && (Boolean) value) {
									g.setColor(Color.GREEN);
								} else {
									g.setColor(Color.RED);
								}
								// Draw circle in center of available space
								int circleDiameter = (width > height) ? height - 2 : width - 2; // Let circle make full use of available space; unusual, but height could be bigger in size then width
								g.fillOval(width / 2 - circleDiameter / 2, height / 2 - circleDiameter / 2, circleDiameter, circleDiameter);
							}
						};
						return panel;
					}
				});		
		table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				_logger.debug("Mouse-Event");
				if (SwingUtilities.isRightMouseButton(e) && e.getComponent() instanceof JTable) {
					int rowNumber = table.rowAtPoint(e.getPoint());
					table.setRowSelectionInterval(rowNumber, rowNumber); // Convenient method for table.getSelectionModel().setSelectionInterval(rowNumber,rowNumber)
					
					NodeTableModel model = (NodeTableModel) table.getModel();
					final Node node = model.nodes.get(rowNumber); // Grab right node based on rowNumber and show appropriate JPopupMenu entries

					// May introduce menus "Show Modules" and "Show Handlers" for each node 
					if (_machineNode.getDispatchers().size() > 0 && node.getDispatchers().size() > 0) { // Local side and remote side have at least one dispatcher
						JPopupMenu menu = null;
						// Both sides (local and remote node) need to have the same handler to make use of it's functionality
						for (final Handler localDispatcher : _machineNode.getDispatchers()) {
							for (final String remoteDispatcherString : node.getDispatchers()) {
								if (localDispatcher.getClass().getCanonicalName().equals(remoteDispatcherString)) {
									if (menu == null) {
										menu = new JPopupMenu("Actions");
									}
									JMenuItem menuItem = null;
									for (final DispatcherAction action : localDispatcher.getActions()) {
										menuItem = new JMenuItem(action.getMenuName());
										menuItem.addActionListener(new ActionListener() {
											@Override
											public void actionPerformed(ActionEvent e) {
												Class cls = localDispatcher.getClass(); // Execute handler action with local handler to interact with remote node
												try {
													final Method method = cls.getDeclaredMethod(action.getSimpleMethodName(), new Class[] { Node.class });
													_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() { // Invoke in separate thread to free AWT-Thread and avoid any blocking of UI
														@Override
														public void run() {
															try {
																method.invoke(localDispatcher, node); // InvocationTargetException
															} catch (IllegalArgumentException e) {
																e.printStackTrace();
															} catch (IllegalAccessException e) {
																e.printStackTrace();
															} catch (InvocationTargetException e) {
																e.printStackTrace();
															}
														}
													});
												} catch (SecurityException e1) {
													e1.printStackTrace();
												} catch (NoSuchMethodException e1) {
													_logger.debug("Could not execute dispatcher action, because dispatcher " + localDispatcher.getClass().getCanonicalName() + " doesn't implement made known method " + action.getSimpleMethodName() + ". This shouldn't happen.");
													e1.printStackTrace();
												} catch (IllegalArgumentException e1) { // Interesting exception handling, why variables name needs to be the same everywhere???
													e1.printStackTrace();
												}
												_logger.debug("User triggered action [" + action.getMenuName() + "] on dispatcher [" + remoteDispatcherString + "] on node [" + node.getIdentifier() + "]");
											}
										});
										menu.add(menuItem);
									}
								}
							}
						}
						if (menu != null) { // Only show menu if there is at least one Dispatcher available; else there is no menu necessary and available as well
							menu.show(e.getComponent(), e.getX(), e.getY()); // Reference 'menu' is null, why?
						}
					}
				}
			}
		});

		JScrollPane tableScrollPane = new JScrollPane(table);
		contentPane.add(tableScrollPane, "grow");
	}
	
	class NodeTableModel extends AbstractTableModel implements NodeListener, ModuleManagerListener {
		private List<Node> nodes = new ArrayList<Node>();
		private String[] columns = new String[] { "", "ID", "Nick" };

		/* Three methods overrides for AbstractTableModel */
		@Override
		public int getColumnCount() {
			return columns.length;
		}

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			Object object = getValueAt(0, columnIndex);
			return object.getClass();
		}

		@Override
		public int getRowCount() {
			return nodes.size();
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			switch (columnIndex) {
			case 0:
				return nodes.get(rowIndex).isAvailable();
			case 1:
				return nodes.get(rowIndex).getNodeId();
			case 2:
				String nick = nodes.get(rowIndex).getNick();
				return (nick == null) ? "" : nick; // Display empty String if nick not set
			}
			return null; // Needed for compiler to calm down, but this line won't never be reached anyway
		}

		@Override
		public String getColumnName(int column) {
			return columns[column];
		}

		@Override
		public void nodeChanged(Node node) { // Called from Node to notify TableModel as NodeListener about changed data, e.g. new lease available etc.; before this the Node was registered with addNode(Node) from outside
			fireTableRowsUpdated(nodes.indexOf(node), nodes.indexOf(node)); // Node can be new or already created, JTable will call getValueAt(nodes.indexOf(node), ...) based on this notification
		}

		@Override
		public void newNode(ModuleManager moduleManager, Node node) { // Called from *ModuleManager* to signal that a new node has been created; MachineNodeGUI takes care of registering this NodeTableModel to ModuleManager
			if (!nodes.contains(node)) { // Shouln't happen that same node get's added multiple times, but why not check for duplicates, and return logger warning when this happened, which shouldn't happen
				nodes.add(node);
				node.addListener(this); // NodeTableModel wants to be notified, when node changed to about values in GUI
				_logger.debug("Tracking new node {}.", node.getNodeId());
				fireTableRowsInserted(nodes.indexOf(node), nodes.indexOf(node)); // Notify TableModelListeners (=JTable) that *new* row was created, which is *not* the same as row update
			} else {
				_logger.debug("ModuleManager tried to include node {} again, which shouldn't happen.", node.getNodeId());
			}
		}
	}
}
