package mygroup.myartifact;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.NoSuchElementException;

import org.slf4j.LoggerFactory;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import ch.qos.logback.classic.Logger;
import ch.qos.logback.classic.encoder.PatternLayoutEncoder;
import ch.qos.logback.core.Context;
import ch.qos.logback.core.FileAppender;
import ch.qos.logback.core.util.StatusPrinter;

/**
 * Class with utility functionality.
 * 
 * @author dell
 * 
 */
public class Utility {
	static SecureRandom secureRandom; // Creating a SecureRandom object is very costly, so generate it once and for all
	static {
		secureRandom = new SecureRandom();
	}
	
	public static String randomId() { // Generate random id <hostname>-<unixtime>-<8_random_chars>
		// Get hostname of local machine; what if there is no hostname? In this case null?
		String hostname = null; // Hostname as distinguishing feature between nodes; how about integrating MAC-address? Another good, maybe even better unique identifier
		try {
			hostname = InetAddress.getLocalHost().getHostName() + "-";
		} catch (UnknownHostException e) { // It is possible that a machine doesn't has a hostname, need to take care of this case
			hostname = "";
			e.printStackTrace();
		}

		// Get current unixtime in seconds, of course of +0000 UTC time; everything else just introduces unnecessary complexity into software
		Long longTime = new Long(new Date().getTime() / 1000); // Seconds since 01.01.1970 GMT=UTC (so no time zone complexity introduced)
		String unixTimeString = longTime + "-";

		// Generate 8 random chars just in case if two computers with same local hostname want to generate an unique identify at the same time
		String randomString = new BigInteger(40, secureRandom).toString(32); // 8 chars, costly because it takes time to generate a SecureRandom object

		return hostname + unixTimeString + randomString;
	}
	
	/*** Various functions to ease working with XML-data ***/
	public static class XML {
		private static org.slf4j.Logger logger = LoggerFactory.getLogger(Utility.XML.class);
		
		// TODO switch to XStream for more flexible XML-serialization solution
		// Helper methods to easily serialize and deserialize object to and from Bean Persistence XML
		public static String serialize(Object object) {
			// Is a lot overhead to create XMLEncoder and XMLDecoder object on every serialization and deserialization? I guess not.
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			XMLEncoder encoder = new XMLEncoder(baos);
			encoder.writeObject(object);
			encoder.close();
			return baos.toString();
		}
		
		// Returns null to signal that something went wrong trying get an object from String; maybe no XML-document, empty String or garbage string data
		//public static Object deserialize(String xml) { // Deserialization could fail, e.g. empty "" String or no xml-representation of object at all
		public static Object deserialize(byte[] xml) {
			ByteArrayInputStream bais = new ByteArrayInputStream(xml);
			XMLDecoder decoder = new XMLDecoder(bais);
			try {
				Object object = decoder.readObject();
				return object;
			} catch (NoSuchElementException e) { // Thrown if msg is empty string "", so need to handle this case
				logger.debug("Could not deserialize XML message to Object. Possible that empty String was send? Message was {}", new String(xml)); // xml.toString()
				e.printStackTrace();
				return null;
			} catch (Exception e) { // I don't know if there are other Exceptions possible, but why not check for this
				logger.debug("Could not deserialize XML message to Object. Message was {}", new String(xml)); // Wieso klappt das mit .toString() nicht? Nur eine Speicheradresse, wie [B@5ae576d4 ???
				e.printStackTrace();
				return null;
			}
		}
	}
	
	/*** IP ***/
	public static class IP {
		public static boolean publicIp() {
			// Two choices to ask, use well-known node if available
			// If not, use wieistmeineip as fallback <- good idea? probably not!
			// Other ways? Don't know, many values could be checked, like if there is a router with UPnP or PMP-NAT; but this would to much right now
			return askWieIstMeineIp();
		}

		public static boolean publicIp(URL url) {
			return askSupernode(url);
		}

		/**
		 * 
		 * @param url
		 *            E.g. http://1.2.3.4:8080/getip
		 * @return
		 */
		private static boolean askSupernode(URL url) {
			// Super-node provides a HTTP-WebServer to easily post a GET request The result is the public IP of this machine.
			// The machine can compare the result with it's local IP-Addressses and determine if this machine has a public IP or not

			// Use Jetty-HTTP-Client or use standard Java classes, no big difference; let's use standard Java classes
			try {
				HttpURLConnection conn = (HttpURLConnection) url.openConnection();
				conn.setRequestMethod("GET");
				// Configure request here if necessary
				conn.connect();
				BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream()));
				String ipStr = br.readLine();
				List<Inet4Address> ip4s = localIPv4addresses();
				for (Inet4Address ip4 : ip4s) {
					if (ip4.getHostAddress().equals(ipStr)) {
						System.out.println("Gefunden, Rechner hat öffentliche IP " + ip4);
						return true;
					}
				}
				System.out.println("Rechner hat KEINE öffentliche IP.");
				return false;
				// return is; // Return as InputStream and then parse content
			} catch (IOException e) {
				System.out.println("Fehler, well-known Supernode anwortet nicht.");
				e.printStackTrace();
				System.out.println("Fallback nutzen.");
				return publicIp();
			}
		}

		private static boolean askWieIstMeineIp() {
			// Make HTTP-request and analyze reponse with linetontains('class="ip"')
			try {
				URL url = new URL("http://www.wieistmeineip.de/");
				URLConnection conn = url.openConnection();
				BufferedReader in = new BufferedReader(new InputStreamReader(conn.getInputStream()));
				String line = null;
				String ipStr = null;
				while ((line = in.readLine()) != null) {
					if (line.indexOf("class=\"ip\"") > 0) { // Looks like: "    <h1 class="ip">92.224.125.114</h1>"
						// Get public IP from service; search for first '>' and first '</'
						int pos1 = line.indexOf(">"); // pos of first '>'
						int pos2 = line.indexOf("</"); // pos of first '</'
						ipStr = line.substring(pos1 + 1, pos2 - 1);
						System.out.println("Öffentliche IP: " + ipStr);
					}
				}

				// If no exception thrown and 'ipStr' set, then we have a public IP we can compare to
				if (ipStr != null) { // HTTP-request was successful, so compare IP with local IPs of interfaces
					List<Inet4Address> ip4s = localIPv4addresses();
					for (Inet4Address ip4 : ip4s) {
						if (ip4.getHostAddress().equals(ipStr)) {
							System.out.println("Gefunden, Rechner hat öffentliche IP " + ip4);
							return true;
						}
					}
					System.out.println("Rechner hat KEINE öffentliche IP.");
					return false;
				}
			} catch (MalformedURLException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return false; // If connection to HTTP-server failed
		}

		private static List<Inet4Address> localIPv4addresses() {
			// Why not return ArrayList, if it also used internally to create a list of known IPv4 addresses?
			List<Inet4Address> ip4s = new ArrayList<Inet4Address>(); // Does an ArrayList has more methods then a List ???
			try {
				Enumeration<NetworkInterface> nets = NetworkInterface.getNetworkInterfaces();
				for (NetworkInterface netint : Collections.list(nets)) { // lo, eth11, net15, ...
					for (Enumeration<InetAddress> e = netint.getInetAddresses(); e.hasMoreElements();) { // IPv4, IPv6 address
						// System.out.println( netint.getName() ); // lo, eth11, net15, ...
						InetAddress inet = e.nextElement();
						if (inet instanceof Inet4Address) {
							Inet4Address ip4 = (Inet4Address) inet;
							// System.out.println("Ist IPv4");
							// System.out.println( netint.getName()+":"+ip4.getHostAddress() ); // gut, ipv4 oder IPv6
							ip4s.add(ip4);
						}
					}
				}
			} catch (SocketException e) {
				e.printStackTrace();
			}
			return ip4s;
		}
	}
}
