package mygroup.myartifact.dispatchers;

/**
 * A dispatcher can supply zero, one, or more actions, which can executed. This reminds me a lot of Beans and their
 * component architecture, and partially it is the same concept.
 * 
 * Basically a DispatcherAction supplies a name for an Action, e.g. "Initiate Chat", which can used to represent the
 * action, e.g. in a JPopupMenu, and a method description, to call a method on a dispatcher to execute the Action.
 * 
 * Unfortunately complete abstract method signatures are not possible and there must be some consensus about what
 * method parameters are supplied. The problem is that the caller, using an object, which is instantiated with
 * reflection, must supply appropriate parameters, which make sense to the called method.
 * 
 * Because of this the list of supplied parameters need to be fixed before-hand. The list of fixed parameters may
 * change in the near future, but currently it is only one parameter:
 *  - Node node: a node object, in which context 
 * 
 * The same is true for return type. The caller needs to know what a specific return type needs to be interpreted, e.g.
 * what a boolean value represents or what information a String contains. Currently void is returned, so no return
 * value at all, which is represented as null returned by Method.invoke().
 * 
 * @author dell
 *
 */
public class DispatcherAction {
	private String simpleMethodName; // Method name of dispatcher action to call this with reflection
	//private Class[] methodSignature; // Useless, because parameters must be fixed before-hand; the same applies to returnType as well
	//private Class returnType;
	private String menuName; // Short, nice-looking string representing this operation in GUI
	
	public DispatcherAction(String simpleMethodName, String menuName) {
		this.simpleMethodName = simpleMethodName;
		this.menuName = menuName;
	}
	
	public String getSimpleMethodName() {
		return this.simpleMethodName;
	}
	
	public String getMenuName() {
		return this.menuName;
	}
	
}
