package mygroup.myartifact.dispatchers.chat;

import java.awt.Color;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import net.miginfocom.swing.MigLayout;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * GUI started, when a chat is initiated. A ChatGUI created and used by ChatProviderGUI. The class ChatGUI is needed as
 * a helper class, because ChatProviderGUI (unfortunately) needs to inherit from ChatProvider, which isn't an
 * interface, but an abstract class, to provider basic functionality.
 * TODO How about changing ChatProvider from abstract class to interface and supply a helper class ChatProviderDefault,
 * which comes with some predefined functionality. This class can also be abstract as well.
 * But in this case no needed constructor can be specified!?! Reflection is the real problem here!!! Maybe this
 * needs to be changed, the current reflection-approach seems to be very fragile.
 * 
 * STOP, THIS CLASS IS OBSOLETE , BECAUSE THERE IS ALREADY ChatProviderGUI! for a graphical chat
 * Yes, it is BUT unfortunately not able to extend ChatProvider; how about making an interface ChatProvider?
 * No, there is a lot functionality a ChatProvider can give. This is really ugly that JFrame inheritance is blocking everything.
 * 
 * TODO bei Chat open-close-open die alten schon geschriebenen Nachrichten wieder anzeigen, wie bei einem echten Chat mit History
 * 
 * @author dell
 */
public class ChatGUI extends JFrame { // SWITCHED FROM extends JFrame TO extends ChatProvider to make life a little more easier
	private Logger _logger;
	// private ChatDispatcher chat; // SWITCH from ChatDispatcher to ChatProvider, because it is the ChatProdiver that receives and sends messages; ChatDispatcher is just router for each ChatProvider
	private ChatProvider _chatProvider;
	
	private JPanel _contentPane;
	private JTextField _textField;
	// private JTextArea textArea;
	// SWITCH from JTextArea to JTextPane to style text
	private JTextPane _textPane;
	private SimpleAttributeSet _meText;
	private SimpleAttributeSet _remoteText;

	public ChatGUI(ChatProvider chatProvider) {
		this(); // Reuse functionality of default constructor, so no need to copy'n'paste the same code again
		this._chatProvider = chatProvider;
		setTitle("Chat with " + chatProvider.partnerId);
	}

	/**
	 * Create the frame. For testing purposes.
	 */
	public ChatGUI() {
		_logger = LoggerFactory.getLogger(ChatGUI.class);
		// TODO possible to use Mocking Framework to test this class
		// setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // Not enough: when click on close also call .stopChat on chat-object; don't know how to handle this yet
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		addWindowListener(new WindowAdapter() { // Close chat session when closing window
			@Override
			public void windowClosing(WindowEvent e) {
				_logger.debug("Chat GUI closing");
				_chatProvider.quitChat();
			}

		});
		setBounds(100, 100, 450, 300);
		_contentPane = new JPanel();
		_contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(_contentPane);
		_contentPane.setLayout(new MigLayout("", "[grow]", "[grow][]"));
		
		_textPane = new JTextPane(); // http://it-republik.de/jaxenter/artikel/Ausgezeichnete-Texte-0364.html
		_textPane.setEditable(false);
		// AttributeSet for own text
		_meText = new SimpleAttributeSet();
		StyleConstants.setForeground(_meText, Color.BLUE);
		StyleConstants.setBold(_meText, true);
		// AttributeSet for remote text
		_remoteText = new SimpleAttributeSet();
		StyleConstants.setForeground(_remoteText, Color.RED);
		StyleConstants.setBold(_remoteText, true);
		_contentPane.add(_textPane, "cell 0 0,grow");

		_textField = new JTextField();
		_textField.addKeyListener(new KeyAdapter() { // Automatically executed in AWT-Thread, so no need to dispatch to AWT-Thread separately
					private List<String> history = new ArrayList<String>();
					private int position = 0;

					@Override
					public void keyReleased(KeyEvent e) {
						int code = e.getKeyCode(); // User may entered text and pressed enter
						if (code == KeyEvent.VK_ENTER) {
							if (_textField.getText().equals("")) { // Don't react on empty message
								return;
							}

							StyledDocument doc = _textPane.getStyledDocument();
							try {
								if (doc.getLength() == 0) {
									doc.insertString(doc.getLength(), "Me: ", _meText);
									doc.insertString(doc.getLength(), _textField.getText(), null);
								} else {
									doc.insertString(doc.getLength(), "\nMe: ", _meText);
									doc.insertString(doc.getLength(), _textField.getText(), null);
								}
							} catch (BadLocationException e1) {
								e1.printStackTrace();
							}
							_chatProvider.tellMessage(_textField.getText());

							// Make history smart: don't save repeated text
							if( history.size()>0 && history.get(history.size()-1).equals(_textField.getText()) ) {
								// Don't save repeated text
							} else {
								history.add(_textField.getText());
								position = history.size(); // Doesn't point on last element, but one shift to the right
							}
							_textField.setText("");
						}

						// History feature
						else if (code == KeyEvent.VK_UP) { // Show previous message in history
							if (history.size() == 0) {
								return;
							} else {
								if (position == 0) {
									position = 0; // Leave position as it is
								} else {
									position = position - 1;
								}
								_textField.setText(history.get(position));
							}
						} else if (code == KeyEvent.VK_DOWN) { // Show next message in history
							if (history.size() == 0) {
								return;
							} else {
								if (position == history.size() || position == history.size() - 1) {
									position = history.size(); // Leave position as it is
									_textField.setText("");
								} else {
									position = position + 1;
									_textField.setText(history.get(position));
								}
							}
						}
					}
				});
		_textField.setColumns(10);
		addWindowFocusListener(new WindowAdapter() { // Give focus to textField when Windows is selected
			@Override
			public void windowGainedFocus(WindowEvent e) {
				_textField.requestFocusInWindow();
			}
		});
		_contentPane.add(_textField, "cell 0 1,growx");
	}

	// Insert new line into JTextArea
	public void appendRemoteText(final String line) {
		// Every GUI-manipulation should be executed in AWT-Thread
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				StyledDocument doc = _textPane.getStyledDocument();
				try {
					if (doc.getLength() == 0) { // This compare feels heavy, maybe anonymous function with changing function could milder this?
						
						//doc.insertString(doc.getLength(), _chatProvider.partnerId + ": ", _remoteText); // Don't show ID, but nickname of remote partner of set; show ID only if there is no nick given
						doc.insertString(doc.getLength(), _chatProvider.getIdentifier() + ": ", _remoteText);
						
						
						doc.insertString(doc.getLength(), line, null);
					} else {
						//doc.insertString(doc.getLength(), "\n" + _chatProvider.partnerId + ": " + _textField.getText(), _remoteText);
						doc.insertString(doc.getLength(), "\n" + _chatProvider.getIdentifier() + ": " + _textField.getText(), _remoteText);
						doc.insertString(doc.getLength(), line, null);
					}
				} catch (BadLocationException e1) {
					e1.printStackTrace();
				}
			}
		});
	}

	public void chatRemotelyQuit() {
		// Remote chat partner quit chat, handle this in GUI, e.g. print message "Partner quit chat" and disable JTextField
		StyledDocument doc = _textPane.getStyledDocument();
		try {
			if (doc.getLength() == 0) { // This compare feels heavy, maybe anonymous function with changing function could milder this?
				doc.insertString(doc.getLength(), "\nPartner quit chat session", null);
			} else {
				doc.insertString(doc.getLength(), "\nPartner quit chat session", null);
			}
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		_textPane.setEnabled(false);
		_textField.setEnabled(false);
	}

	// HANDLING CHAT REQUESTS BY ChatRequestHandler classes; and SENDING AND RECEIVING AND HANDLING MESSAGES IN CHAT BY ChatProvider classes
	// @Override Used previously as part of ChatProvider, but interface removed, because no Strategy Pattern used at all, so why pollute code with new interface?
	public boolean wantToStartChat(String nodeName) {
		// Show user Popup asking if it's OK to start a chat with another node;
		String theMessage = "Do you want to chat with " + nodeName + "?";
		int result = JOptionPane.showConfirmDialog((Component) null, theMessage, "Chat request", JOptionPane.OK_CANCEL_OPTION);
		if (result == JOptionPane.OK_OPTION) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Launch the application. Solely for testing.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					ChatGUI frame = new ChatGUI();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
}
