package mygroup.myartifact.dispatchers.chat;

import mygroup.myartifact.dispatchers.chat.messages.ChatMessage;
import mygroup.myartifact.dispatchers.chat.messages.TellMessage;

/**
 * This ChatProvider provides a GUI for a user to easily send and receive message with another chat dispatcher. It 
 * shows a GUI to input new message to other chat dispatcher and to read messages from other chat dispatchers.
 * 
 * The ChatDispatcher in front of this ChatProvider handles routing messages to this ChatProvider and also proceeds the messages
 * to call the appropriate methods. So this ChatProviderGUI doesn't need to be smart figuring what is going on, but just needs
 * to implement the apropriate method bodies.
 * 
 * This class uses an internal instance of ChatGUI to display a GUI.
 * 
 * @author dell
 *
 */
public class ChatProviderGUI extends ChatProvider {
	private ChatGUI gui;

	public ChatProviderGUI(ChatHandler chatDispatcher, String partnerId) {
		super(chatDispatcher, partnerId);
		gui = new ChatGUI(this);
		gui.setVisible(true);
	}
	
	@Override
	public void chatRemotelyQuit() {
		super.chatRemotelyQuit();
		// Remote chat partner quit chat, handle this in GUI, e.g. print message "Partner quit chat" and disable JTextField
		gui.chatRemotelyQuit();
	}
	
	@Override
	public void messageReceived(TellMessage tellMessage) {
		// Remote chat partner sent a chat message, print this message to the user
//		String line = tellMessage.getSender()+": "+tellMessage.getText();
		gui.appendRemoteText(tellMessage.getText());	
	}
}
