package mygroup.myartifact.dispatchers.chat;

import mygroup.myartifact.dispatchers.chat.messages.TellMessage;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A simple chat provider that reacts on incoming chat messages by sending back the same text with a simple prefix and suffix text. So this is a very annoying chat provider.
 * 
 * This chat provider never quits a chat session.
 * TODO how about this chat provider automatically quits a chat session when remote side didn't send a message for 15 seconds?
 * 
 * @author dell
 *
 */
public class ChatProviderRepeat extends ChatProvider {
	private Logger logger;

	public ChatProviderRepeat(ChatHandler chatDispatcher, String partnerId) {
		super(chatDispatcher, partnerId);
		this.logger = LoggerFactory.getLogger(ChatProviderRepeat.class);
		
		// Automatically quit chat session, when user didn't send any message for 20 seconds
	}

	@Override
	public void messageReceived(TellMessage tellMessage) {
		logger.debug("Received message from remote node {}, message is {}.", new Object[]{tellMessage.getSender(), tellMessage.getText()});
		// Send back same message with annoying prefix and suffix
		String annoying = "What do you mean with \""+tellMessage.getText()+"\"? I don't understand you, please be more clear!";
		//TellMessage tell = new TellMessage(tellMessage.getReceiver(), partnerId, "Annoying"); // Is there a better way to get the ID of this machineNode? Better way to create messages?
		TellMessage tell = new TellMessage(chatDispatcher.getLocalNodeId(), partnerId, annoying);
		logger.debug("Sending back annoying message.");
		sendChatMessage(tell); //chatDispatcher.sendChatMessage(tell);
	}

	@Override
	public void chatRemotelyQuit() {
		super.chatRemotelyQuit();
		logger.debug("Remote node {} quit chat session.", partnerId);
	}
	
}
 