package mygroup.myartifact.dispatchers.ping;

import java.awt.GraphicsEnvironment;
import java.util.Properties;

import javax.swing.JOptionPane;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import mygroup.myartifact.MachineNode;
import mygroup.myartifact.dispatchers.Handler;
import mygroup.myartifact.dispatchers.DispatcherAction;
import mygroup.myartifact.dispatchers.ping.messages.PingPingMessage;
import mygroup.myartifact.dispatchers.ping.messages.PingPongMessage;
import mygroup.myartifact.modules.Node;
import mygroup.myartifact.modules.messages.NodeMessage;
import mygroup.myartifact.modules.messages.UnicastMessage;

/**
 * A very simple dispatcher to to send a PING to a remote node and get a PONG as response.
 * 
 * This dispatcher was initially introduced to test the request-response feature on node-level.
 * 
 * @author dell
 * 
 */
public class PingHandler extends Handler {
	private Logger _logger;
	
	public PingHandler(MachineNode machineNode, Properties properties) {
		super(machineNode, properties);
		_logger = LoggerFactory.getLogger(PingHandler.class);
	}

	// Received NodeMessage from MachineNode to try to proceed the message; return false if wasn't able to proceed the message
	@Override
	public boolean dispatchNodeMessage(NodeMessage nodeMessage) {
		if (nodeMessage instanceof PingPingMessage) {
			_logger.debug("Received PING from node {}, sending back PONG.", nodeMessage.getSender());
			PingPongMessage pongMessage = new PingPongMessage(_machineNode.getId(), nodeMessage.getSender(), nodeMessage.getMessageId());
			_machineNode.sendNodeMessage(pongMessage);
			return true;
		}
		return false;
	}
	
	public void ping(final Node remoteNode) {
		PingPingMessage pingRequest = new PingPingMessage(_machineNode.getId(), remoteNode.getNodeId());
		_logger.debug("Sending PING with message-id {} to node {}", new Object[]{ pingRequest.getMessageId(), remoteNode.getNodeId() });
				
		final long startTime = System.currentTimeMillis();
		UnicastMessage unicastMessage = _machineNode.sendNodeRequestMessage(pingRequest); // The following method is blocking for maximal 5 seconds; will return received response as UnicastMessage or null if response wasn't received
		
		if (unicastMessage != null) {
			_logger.debug("Received PONG from node {} after {} miliseconds.", new Object[]{ remoteNode.getNodeId(), System.currentTimeMillis()-startTime });
			if (!GraphicsEnvironment.isHeadless()) {
				_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() {
					@Override
					public void run() {
						JOptionPane.showMessageDialog(null, "Received PONG from node " + remoteNode.getIdentifier() + " after "+(System.currentTimeMillis()-startTime)+" miliseconds.");
					}
				});
			}
		} else {
			_logger.debug("PONG not received, timout reached, aborting waiting any longer.");
			if (!GraphicsEnvironment.isHeadless()) {
				_machineNode.getModuleManager().getExecutorSerivce().submit(new Runnable() {
					@Override
					public void run() {
						JOptionPane.showMessageDialog(null, "Failed to receive PONG from node " + remoteNode.getNodeId() + "."); //	JOptionPane.showMessageDialog(null, "Failed to receive PONG from node " + remoteNode.getIdentifier() + ".");
					}
				});
			}
		}
	}
	
	@Override
	public String getIdentifier() {
		return "Ping";
	}
	
	@Override
	public DispatcherAction[] getActions() {
		DispatcherAction[] dispatcherActions = new DispatcherAction[] { new DispatcherAction("ping", "Ping") };
		return dispatcherActions;
	}
}
