package mygroup.myartifact.dispatchers.transfer;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.table.AbstractTableModel;

import mygroup.myartifact.modules.Node;
import net.miginfocom.swing.MigLayout;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TransferGUI extends JFrame {
	private boolean _debug = true; // A flag to switch between production and run-main mode to isolated test this class

	private Logger _logger;

	private TransferGUI _this = this; // Reminds me a lot about closures and constructor functions in JavaScript
	private TransferHandler _transferDispatcher;
	private Node _remoteNode;

	// TODO Introduce cancel button to abort sending of many and/or large files
	private JPanel _contentPane;
	private JTable _filesTable;
	private JButton _chooseButton;
	private JButton _sendButton;
	private TableModel _tableModel;
	private JProgressBar _progressBar;
	private JLabel _progressLabel;
	
	public TransferGUI(TransferHandler transferDispatcher, Node remoteNode) {
		this(); // Needed to call this explicitly to load all GUI-elements
		_transferDispatcher = transferDispatcher;
		_remoteNode = remoteNode;

		_debug = false; // Class is in production environment
	}

	/**
	 * Create the frame.
	 */
	public TransferGUI() {
		try {
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"); // Works only with Sun/Oracle JVM
		} catch (Exception e) {
			e.printStackTrace();
		}
		_logger = LoggerFactory.getLogger(TransferGUI.class);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 450, 300);
		_contentPane = new JPanel();
		_contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(_contentPane);
		_contentPane.setLayout(new MigLayout("", "[grow][]", "[][grow][][]"));

		_chooseButton = new JButton("Choose files(s)");
		_chooseButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent mouseEvent) {
				if (SwingUtilities.isLeftMouseButton(mouseEvent)) {
					JFileChooser fileChooser = new JFileChooser();
					fileChooser.setMultiSelectionEnabled(true);
					int returnValue = fileChooser.showOpenDialog(_this);
					if (returnValue == JFileChooser.APPROVE_OPTION) {
						_sendButton.setEnabled(true);
						_tableModel.addFiles(fileChooser.getSelectedFiles());
					}
				}
			}
		});
		_contentPane.add(_chooseButton, "cell 0 0");

		_sendButton = new JButton("Send file(s)");
		_sendButton.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent mouseEvent) {
				if (SwingUtilities.isLeftMouseButton(mouseEvent)) {
					// Deactivate UI elements
					_chooseButton.setEnabled(false);
					_sendButton.setEnabled(false);
					_filesTable.setEnabled(false);

					// Connect to remote node
					_progressLabel.setText("Establishing direct connection with remote node " + _remoteNode + " ...");
					_logger.debug("Establishing direct connection with remote node {} ...", _remoteNode);
					Socket socket = null;
					if (!_debug) {
						socket = _remoteNode.getConnection(_transferDispatcher.getClass().getCanonicalName());
					} else {
						// Make debug-mode very easy: assume a ServerSocket is already running on localhost:13444, connect to this and start transfering files
						try {
							socket = new Socket("localhost", 13444);
						} catch (UnknownHostException e) {
							e.printStackTrace();
						} catch (IOException e) {
							e.printStackTrace();
						}
					}

					if (socket == null) {
						_logger.debug("Failed to establish direct connection to remote node {}, can't send files.", _remoteNode);
						_progressLabel.setText("Failed to establish direct connection to remote node " + _remoteNode + ", can't send files.");
					} else {
						_progressLabel.setText("Established direct connection with remote node " + _remoteNode);
						_logger.debug("Established direct connection with remote node {}", _remoteNode);

						// Transfer/Upload files to remote node
						byte[] buffer = new byte[1024 * 32]; // Take care of RAM by using buffer
						// InputStream is = null;
						List<File> files = _tableModel.getFiles();

						// TODO send *both*, FileInformation-object and file itself
						try {
							ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
							for (File file : files) {
								_progressLabel.setText("Sending file " + file.getName());
								_logger.debug("Sending file {} with {} byte filesize.", new Object[] { file.getName(), file.length() });
								try {
									InputStream is = new FileInputStream(file);

									// Send FileInformation
									FileInformation fileInformation = new FileInformation(file.getName(), file.length());
									oos.writeObject(fileInformation);
									oos.flush();
									_logger.debug("Sent FileInformation");

									// Send file itself
									int bytesRead = 0;
									while ((bytesRead = is.read(buffer)) > 0) { // Check !=-1 or >0 more robust? http://stackoverflow.com/questions/277260/what-is-the-best-way-to-transfer-a-file-using-java
										_logger.debug("Read {} byte from file", bytesRead);
										// TODO using direct OutputStream.write(...) writes one byte at a time byte, not efficient; use Buffered-version for more performance
										// socket.getOutputStream().write(buffer, 0, bytesRead); // Connection reset by peer!?! Why should TransferDispatcher.handleConnection() close socket?
										socket.getOutputStream().write(buffer, 0, bytesRead); // Connection reset by peer!?! Why should TransferDispatcher.handleConnection() close socket?
										_logger.debug("Wrote {} byte to socket", bytesRead);
									}
								} catch (FileNotFoundException e) {
									_logger.debug("Could not load file {}, skipping to next file.", file);
									e.printStackTrace();
									continue;
								} catch (IOException e) {
									_logger.debug("Could not transfer file {} due to arbitrary IO error, skipping to next file.", file);
									e.printStackTrace();
									continue;
								}
							}
							_progressLabel.setText("All files sent");
							_logger.debug("Processed all files.");
						} catch (IOException e1) {
							_logger.debug("Could not get outputstream to write to socket, aborting file transfer.");
							e1.printStackTrace();
						} finally {
							try {
								socket.close();
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
					}
				}
				// Bring back UI elements
				_chooseButton.setEnabled(true);
				_filesTable.setEnabled(true);
			}
		});
		_sendButton.setEnabled(false);
		_contentPane.add(_sendButton, "cell 0 0");
		
		_tableModel = new TableModel();
		_filesTable = new JTable(_tableModel);
		_filesTable.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				if (SwingUtilities.isRightMouseButton(e) && e.getComponent() instanceof JTable) {
					JPopupMenu menu = new JPopupMenu();
					JMenuItem removeItem = new JMenuItem("Remove");
					removeItem.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent actionEvent) {
							_tableModel.removeFiles(_filesTable.getSelectedRows());
						}
					});
					menu.add(removeItem);
					menu.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});
		JScrollPane scrollPane = new JScrollPane(_filesTable);
		_contentPane.add(scrollPane, "cell 0 1,grow"); // contentPane.add(filesTable, "grow");

		_progressBar = new JProgressBar();
		_contentPane.add(_progressBar, "cell 0 2");

		_progressLabel = new JLabel("Choose one or more files and click on Send file(s)");
		_contentPane.add(_progressLabel, "cell 0 3");
	}

	// JTable uses TableModel to retrieve values;
	class TableModel extends AbstractTableModel {
		private List<File> _files = new ArrayList<File>(); // Like in Scala
		private String[] _columns = new String[] { "Files" };

		// To add, remove, and get files to TableModel
		public void addFiles(File[] files) {
			for (File file : files) {
				if (!_files.contains(file)) {
					_files.add(file);
					fireTableRowsInserted(_files.indexOf(file), _files.indexOf(_files));
				}
			}
		}

		public void removeFiles(int[] filePositions) {
			Arrays.sort(filePositions);
			for (int i = filePositions.length - 1; i >= 0; i--) {
				_files.remove(filePositions[i]);
				fireTableRowsDeleted(filePositions[i], filePositions[i]);
			}
		}

		public List<File> getFiles() {
			return _files;
		}

		// Called by JTable to retrieve values
		@Override
		public String getColumnName(int column) {
			return _columns[column];
		}

		@Override
		public int getColumnCount() {
			return 1;
		}

		@Override
		public int getRowCount() {
			return _files.size();
		}

		@Override
		public Object getValueAt(int rowIndex, int columnIndex) {
			return _files.get(rowIndex);
		}

		@Override
		public Class<?> getColumnClass(int columnIndex) {
			return getValueAt(0, columnIndex).getClass();
		}
	}

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		System.out.println("Running TransferGUI in debug-mode, current System.getProperty('user.dir') is "+System.getProperty("user.dir"));
		
		// Start GUI in debug-mode
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					TransferGUI frame = new TransferGUI();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
}
