package mygroup.myartifact.modules;

import java.net.Socket;
import java.util.Properties;
import java.util.concurrent.Future;

import mygroup.myartifact.modules.messages.NodeMessage;

/**
 * A module uses a specific transport mechanism and implements a custom strategy to find other nodes.
 * 
 * A module needs to be smart when it comes to find new nodes and to notice if a node is not available anymore.
 * A module may uses a PUSH mechanism to notify about those changes.
 * 
 * @author dell
 */
public abstract class Module {	 
	protected Properties _properties;
	protected ModuleManager _moduleManager;
	protected int _priority; // Used by Node (previously by ModuleManager) to determine which module to use for a specific operation, effectively to send a message
	protected boolean _running = false; // Maybe introduce more states if needed, e.g. 'sleep', 'waiting' etc.
	
	public Module(Properties properties, ModuleManager moduleManager, int priority) {
		_properties = properties;
		_moduleManager = moduleManager;
		_priority = priority;
	}
	
	public int getPriority() {
		return _priority;
	}
	
	abstract public String getIdentifier(); // Let module identify itself with a simple text, e.g. "Multicast", "Websocket", "Bluetooth" etc.
	
	public abstract boolean start(); // Call to start module; returns true if successfully started, false if module couldn't be started
	public abstract void stop();     // Call to stop module
	
	abstract public void sendNodeMessage(NodeMessage nodeMessage);
	
	@Override
	public String toString() {
		return getIdentifier()+" ("+this.getClass()+")"; // 'this' can deal with subclasses, so works as expected and returns package-name of subclass
	}
	
	// Try to establish connection with remote-node; currently only TCP-connections supported, in future also UDP and SCTP (can wrap UDP-connection etc.)
    //abstract public Future<Socket> createDirectConnection(Node remoteNode);
	abstract public Socket getConnection(Node remoteNode, String dispatcherClass);
}
