package mygroup.myartifact.modules.global.supernode.webservice;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;


public class ContextListener implements ServletContextListener {
	private int scheduleTime = 5;
	private boolean schedule;
	
	// http://stackoverflow.com/questions/3084542/background-process-in-servlet
	private ScheduledExecutorService scheduler;
	private ScheduledFuture<?> handler;
	
	public ContextListener(boolean schedule) {
		this.schedule = schedule;
		System.out.println("##### ContextListener Constructor");
	}

	public void contextInitialized(ServletContextEvent e) {
		System.out.println("ContextListener: contextInitialized");
		// TODO some mechanism to load the list contents from a persistent source
		
		Map<String, Peer> peers = new ConcurrentHashMap<String, Peer>(); //Map<String, Peer> peers = Collections.synchronizedMap(new HashMap<String, Peer>());
		
		long leaseTime = 10;
		Peer[] dummyPeers = null;
			dummyPeers = new Peer[] {
					new Peer("abc", "joe", "1.2.3.4", System.currentTimeMillis() / 1000 + leaseTime + 10),
					new Peer("xyz", "joe", "5.6.7.8", System.currentTimeMillis() / 1000 + leaseTime + 5),
					new Peer("123", "joe", "9.10.11.12", System.currentTimeMillis() / 1000 + leaseTime + 12) };
		for (Peer peer : dummyPeers) {
			peers.put(peer.getId(), peer);
		}
		System.out.println("ContextListener: Peers is " + peers); // empty
		
		e.getServletContext().setAttribute("peerlist", peers); // Store list in servlet context
		
		/* Activate background thread to update peer list according to lease time */
		scheduler = Executors.newSingleThreadScheduledExecutor();
		PeerListUpdater updater = new PeerListUpdater(peers);
		// The executor will start after 0 seconds (=TimeUnit.SECONDS) and repeat every second
		if(schedule) {
			handler = scheduler.scheduleAtFixedRate(updater, 0, this.scheduleTime, TimeUnit.SECONDS); // executes updater.run() every this.scheduleTime seconds
			// you can use the handler to cancel the repeated execution of updater.run();
			// or the termination of the executor (introduces externally) cancels the period execution
			
			// TODO of course it would be better not to execute this every second, but only when needed, this is when a leaseTime ends;
			// but this is not important in the moment
		} // Don't schedule automatic peer list updater, if 'schedule' is set to false
		
	}

	public void contextDestroyed(ServletContextEvent e) {
		System.out.println("ContextListener: contextDestroyed");
		// TODO some mechanism to store the list contents persistently
	}
}
