package mygroup.myartifact.modules.global.supernode.webservice;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.Produces;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

@Provider
// Mark this class, so Jersey knows to treat this class as a new
// MessageBodyWriter; 'implements' not enough?
@Produces("application/xml")
// This writer is only able to generate XML data as media type
public class PeerListMessageBodyWriter implements MessageBodyWriter<List<Peer>> {

	public PeerListMessageBodyWriter() {
		System.out.println("=== === === === ===");
		System.out.println("PeerListMessageBodyWriter: instantiated");
		System.out.println("=== === === === ===");
	}

	/**
	 * Called by JAX-RS to determine if this writer is able to marshall this
	 * object.
	 * 
	 * @param t
	 *            The java.lang.Class type of the object to be marshalled. Type
	 *            is determined by calling getClass() on the object.
	 * @param type
	 *            The java.lang.reflect.Type for generic type information
	 *            (generics) about the object. Useful for List<...> etc.
	 * @param annotation
	 *            Array of java.lang.annotation.Annotation; I don't what this is
	 *            good for
	 * @param mediaType
	 *            The media type that our JAX-RS method wants to produce.
	 */
	public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
		// Does this MessageBodyWriter supports marshalling of 'this' type?

		// Ensure that we're handling only List<GPSTrackerCollection> objects.
		boolean isWritable;
		if (List.class.isAssignableFrom(type) && genericType instanceof ParameterizedType) {
			ParameterizedType parameterizedType = (ParameterizedType) genericType;
			Type[] actualTypeArgs = (parameterizedType.getActualTypeArguments());
			isWritable = (actualTypeArgs.length == 1 && actualTypeArgs[0].equals(Peer.class)); // yes,
																								// is
																								// List<Peer>
		} else {
			isWritable = false;
		}

		return isWritable;
	}

	public long getSize(List<Peer> t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
		// Called to determine the Content-Length of the output
		return -1; // Let the underlying HTTP layer (i.e., a servlet container)
					// determine the Content-Length of the output.
	}

	public void writeTo(List<Peer> list, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType,
			MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException, WebApplicationException {
		// Does the heavy lifting and writes the content out to the HTTP
		// response buffer

		/* Build DOM */
		// Create XML
		// http://www.genedavis.com/library/xml/java_dom_xml_creation.jsp
		Document doc = null;
		try {
			// Create document
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
			doc = docBuilder.newDocument(); // org.w3c.dom.Document

			// Create root element
			Element peersTag = doc.createElement("peers");
			doc.appendChild(peersTag);

			// Create peer element for each peer in list
			for (Peer peer : list) {
				Element peerTag = doc.createElement("peer");

				// Peer id
				Element idTag = doc.createElement("id");
				Text idTxt = doc.createTextNode(peer.getId());
				idTag.appendChild(idTxt);
				peerTag.appendChild(idTag);

				// Only create nickTag if nick is not null
				if (peer.getNick() != null) {
					// Peer nick
					Element nickTag = doc.createElement("nick");
					Text nickTxt = doc.createTextNode(peer.getNick());
					nickTag.appendChild(nickTxt);
					peerTag.appendChild(nickTag);
				}

				// Peer globalIp
				Element globalIpTag = doc.createElement("globalIp");
				Text globalIpTxt = doc.createTextNode(peer.getGlobalIP());
				globalIpTag.appendChild(globalIpTxt);
				peerTag.appendChild(globalIpTag);

				// Peer leaseTime
				Element leaseTimeTag = doc.createElement("leaseTime");
				Text leaseTimeTxt = doc.createTextNode(String.valueOf(peer.getLeaseTime()));
				leaseTimeTag.appendChild(leaseTimeTxt);
				peerTag.appendChild(leaseTimeTag);

				// Append new <peer> node into parent node <peers>
				peersTag.appendChild(peerTag);
			}
		} catch (ParserConfigurationException e) {
			System.out.println("This should *never* happen.");
			System.exit(1);
			e.printStackTrace();
		}

		/* Build Transformer to output DOM representation to XML */
		Transformer trans = null;
		try {
			TransformerFactory transfac = TransformerFactory.newInstance();
			trans = transfac.newTransformer();
			// TODO the generated XML file looks kinda ugly: no <?xml...?> head
			// at the beginning
			// also no beautified output and unnecessary \n in the generated
			// document
			trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
			trans.setOutputProperty(OutputKeys.INDENT, "yes");
		} catch (TransformerConfigurationException e) {
			System.out.println("This should *never* happen.");
			System.exit(1);
			e.printStackTrace();
		}

		/* Print XML to OutputStream */
		try {
			StreamResult result = new StreamResult();
			result.setOutputStream(entityStream);
			trans.transform(new DOMSource(doc), result);
		} catch (TransformerException e) {
			e.printStackTrace();
		}

	}

	/* Test method to see if this class produces correct output */
	public static void main(String[] args) {	
		PeerListMessageBodyWriter writer = new PeerListMessageBodyWriter();
		// Input: List<Peer>, everything else can be safely set to null
		List<Peer> list = new ArrayList<Peer>();
		list.add(new Peer());

		long leaseTime = 10;
		List<Peer> dummyPeers = new ArrayList<Peer>();
		dummyPeers.add( new Peer("abc", "joe", "1.2.3.4", System.currentTimeMillis() / 1000 + leaseTime + 10) );
		dummyPeers.add( new Peer("xyz", "joe", "5.6.7.8", System.currentTimeMillis() / 1000 + leaseTime + 5) );
		dummyPeers.add( new Peer("123", "joe", "9.10.11.12", System.currentTimeMillis() / 1000 + leaseTime + 12) );
		
		// Output: the constructed XML-document
		try {
			writer.writeTo(dummyPeers, null, null, null, null, null, System.out); // write output directly to console
		} catch (WebApplicationException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
}
