package mygroup.myartifact.modules.global.supernode.webservice;

import java.util.Map;

/**
 * This Runnable object gets periodically executed.
 * 
 * @author daniel
 */
public class PeerListUpdater implements Runnable {
	Map<String, Peer> peers;
	
	public PeerListUpdater(Map<String, Peer> list) {
		this.peers = list;
	}
	
	public void run() {
		System.err.println("### PeerListUpdater: enter run() method");
		System.out.println("Current list of known peers is:");
		for(String id : peers.keySet()) {
			Peer peer = peers.get(id);
			System.out.println("  "+peer+", "+(peer.getLeaseTime()-System.currentTimeMillis()/1000)+" seconds left");
		}
		
		// Check every Peer-object in list and remove every Peer-object which
		// lease time expired
		// Maybe an error occurs; why is there no exception thrown?
		try {
			boolean flag = false;
			for (String id : peers.keySet()) {
				Peer peer = peers.get(id);
				if (isExpired(peer)) {
					flag = true;
					peers.remove(id);
					System.out.println("Removed peer " + peer.getId() + ", because the lease time expired.");
				}
			}
			if(flag) {
				System.out.println();
				System.out.println("List of known peers changed. Current list is:");
				for(String id : peers.keySet()) {
					Peer peer = peers.get((id));
					System.out.println(peer);
					//System.out.println("  Peer: "+peer.getId()+", "+peer.getNick()+", "+peer.getGlobalIP());
				}
			}
			System.out.println();
		} catch (Exception e) {
			System.out.println("Exception geworfen: " + e); // Sollte nie eine RuntimeException geworfen werden, aber wer weiß schon
			e.getStackTrace();
		}
	}
	
	/**
	 * This methods checks if the registration of the given <code>peer</code>
	 * expired. If yes, then this peer should be removed from the list of known
	 * or registered peers.
	 * 
	 * @param peer
	 * @return true if the registration of <code>peer</code> expired.
	 */
	private boolean isExpired(Peer peer) {
		boolean result = false;
		long lease = peer.getLeaseTime(); // already divided by 1000 !!!
		long now = System.currentTimeMillis() / 1000;
		
//		System.out.println("Leasetime is:       "+lease);
//		System.out.println("Now is:             "+now);
//		System.out.println("Leasetime - now is: "+(lease-now));
		
		if (lease - now < 0) {
			result = true;
		}
		return result;
	}
	
	// // Garbage
	// for(Peer peer : peers.values()) {
	// if(isExpired(peer)) { // remove this peer from list
	// peers.remove(peer);
	// System.out.println("Removed peer "+peer.getId()+", because the lease time expired.");
	// }
	// }
	
}
