package mygroup.myartifact.modules.global.supernode.webservice;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.GenericEntity;
import javax.ws.rs.core.Response;
import javax.xml.bind.annotation.XmlRootElement;

@Path("peers")
//@Singleton // Jersey specific class to instantiate this web service only once
/* I can't use @Singleton because it messes up with @Context-based field injections.
 * For example '@Context ServletRequest r;' is injected with reflection *not* with
 * Method.invoke, but with Field.setValue. And those is a field, so an object instance
 * attribute. I am not 100% sure, why @Singleton with @Context messes up, but it makes
 * sense, because injection always happens *after* constructor call.
 */

/*
 * Overview of test methods:
 * Add
 * 	curl -v -X POST -d "id=unique" http://localhost:8282/rest/peers/add
 * 	curl -v -X POST -d "id=unique" -d "nick=joe" http://localhost:8282/rest/peers/add
 * Delete
 * 	curl -v -X DELETE -d "id=unique" http://localhost:8282/rest/peers/delete
 * List
 * 	GET: curl -v http://localhost:8282/rest/peers/list
 * 
 */
public class PeersWebService {
	//@Context // Seems like ServletContext is only available as parameter, not as object attribute throw reflection injection
	private ServletContext context; // Set in constructor
	private long leaseTime = 10; // a peer needs to renew his registration every 10 seconds (short time for testing and debugging 
	
	// TODO take care of duplicates: switch from HashMap to HashSet (easy) or check for duplicates (hard)
	private Map<String,Peer> peers; // this collection is externalized into the servlet context 
	
	public PeersWebService(@Context ServletContext c) { // Constructor gets executed for *each* request; there is no @Singleton annotation at class level
		this.context = c; // Get ServletContext through reflection injection
		
		// Gibt es doch nicht: immer noch ist obj ein null pointer. Wie kann das sein?
		//Object obj = context.getAttribute("peerlist"); // What's wrong here?
		//peers = (Map<String,Peer>) obj; // Get reference to list of peers in servlet context
		
		peers = (Map<String,Peer>) context.getAttribute("peerlist");
	}
	
	@GET
	@Path("list")
	@Produces("application/xml")
	/**
	 * Test with:
	 * curl -v http://localhost:8282/rest/peers/list
	 */
	//public ListBean list() {
	public List<Peer> list() {
	//public Response list() {
		List<Peer> list = new ArrayList<Peer>(peers.values()); // copy of references or wrapper-approach? don't know, hope it works	
		//ListBean<Peer> listBean = new ListBean<Peer>(list);
		
		// http://markmail.org/message/t6r34qravqsowxx5#query:+page:1+mid:dqnjmbdygpr5g3rx+state:results
		// http://jsr311.java.net/nonav/javadoc/javax/ws/rs/core/GenericEntity.html
		
		// TODO why is list empty?
		
		
		//List<String> list = new ArrayList<String>();
		
		//GenericEntity<ListBean<Peer>> entity = new GenericEntity<ListBean<Peer>>( listBean) {};
		// Person with the same problem http://christopherhunt-software.blogspot.com/2010/08/messagebodywriter-iswriteable-method.html
		
		
		//GenericEntity<List<Peer>> entity = new GenericEntity<List<Peer>>( list) {}; // Preserve information on the parametrized type, because the runtime strips this information (this is called erasure)
		//Response response = Response.ok(entity).build();
		//return response;
		
		
		return list; // JAX-RS automatically find, registers, and uses PeerListMessageBodyWriter to produce the output for this method
		//return listBean; // Create new local wrapper-class to return list of peers
		// TODO include URIs to peer resources with UriBuilder
	}	
	
	@Path("add")
	@POST
	//@Produces("text/plain")
	@Produces("application/xml")
	/** 
	 * Test with:
	 * curl -v -d "id=unique" http://localhost:8282/rest/peers/add
	 * curl -v -X POST -d "id=unique" http://localhost:8282/rest/peers/add
	 * curl -v -X POST -d "id=unique" -d "nick=joe" http://localhost:8282/rest/peers/add
	 * 
	 * (More RESTful(???): curl -X POST -d @filename http://example.com/path/to/resource)
	 */
	// TODO like in getAdd introduce parameters for leaseTime and globalIp
	public Response add(
			@FormParam("id") String id, // Client needs to set an appropriate unique identifier
			@DefaultValue("") @FormParam("nick") String nick,
			@Context HttpServletRequest request) { // Is 'HttpServletRequest' a parameter to @Context? NO It's a marker, read by reflection to know where to inject a parameter dependency. It's so simple and clear :)
		// id *needs* to be set, otherwise something is completely wront
		// return a "Request ignored, because id was not set."
		if(id==null) {
			return new Response("Request ignored, because id was not set.");
		}
		
		if(nick.equals(""))
			nick = null;
		
		long lease = System.currentTimeMillis()/1000+this.leaseTime;
		Peer peer = new Peer(id, nick, request.getRemoteAddr(),lease);
		
		// Add new peer object
		if(peers.containsKey(id)) { // id is null, why?
			peers.put(id, peer); // here is a problem, NullPointerException;
			return new Response("Peer overwritten.");
			/*returns XML document
			 * <?xml version="1.0" encoding="UTF-8" standalone="yes"?>
			 * <response>
			 *   <result>Peer created.</result>
			 * </response>
			 */
		} else {
			peers.put(id, peer);
			return new Response("Peer created."); //return XML document
		}
		
		
		// TODO What if peer with this id is already present?
		//      For the moment: just overwrite and maybe return a message like "Peer overwritten."
	}
	
	@GET
	@Path("get/{id}")
	/**
	 * Get peer based on id String.
	 * Test with:
	 * curl -v http://localhost:8282/rest/peers/get/{id}
	 */
	
	public Peer get(@PathParam("id") String id) {
		return peers.get(id); // maybe null, but I don't think this is a problem
	}
	
	@DELETE
	@Path("delete/{id}")
	/** 
	 * Test with:
	 * curl -v -X DELETE -d "id=unique" http://localhost:8282/rest/peers/delete <- OLD, WRONG WAY
	 * curl -v -X DELETE http://localhost:8282/rest/peers/delete/{id} <- NEW, CORRECT WAY
	 */
	// DELETE only works with the supplied URI; no POST-parameters need to be send to make a DELETE-method work
	// I mean look at a HTTP-Request: it consists of HEAD and optional BODY; and the HEAD begins with 'GET', 'DELETE',
	// any other HTTP-method name
	// So change it: go away from POST-data, and get the id from the {id} in the path
	// How could I forget that: @QueryParam is for POST-data, @PathParam for query-data
	public Response delete(@PathParam("id") String id) {
		peers.remove(id); // again a NullPointerException, probably because id wasn't set to a value
		return new Response("Peer deleted.");
	}
	
	/*** Extra GET-based methods to create new peers; this is not very REST-like, but enables easy testing and debugging with a browser ***/
	
	@GET
	@Path("getDelete")
	/**
	 * Test with:
	 * http://localhost:8282/rest/peers/getDelete?id=myid
	 */
	public Response getDelete(@QueryParam("id") String id) {
		peers.remove(id);
		return new Response("Peer deleted.");
	}
	
	@GET
	@Path("getAdd")
	/**
	 * Create new peer with a GET-request
	 * Test in Browser with:
	 * -only id given; nick set to null, lease-time set with default value, ip derived from browser ip
	 * 		http://localhost:8282/rest/peers/getAdd?id=myid
	 * -id and nick given; lease-time set with default value, ip derived from browser ip
	 * 		http://localhost:8282/rest/peers/getAdd?id=myid&nick=mynick
	 * -id, nick and ip given; only lease-time set with default value
	 * 		http://localhost:8282/rest/peers/getAdd?id=myid&nick=mynick&ip=1.2.3.4
	 * -id, nick, ip, and lease given in seconds left
	 * 		http://localhost:8282/rest/peers/getAdd?id=myid&nick=mynick&ip=1.2.3.4&lease=150
	 */
	public Response getAdd(
			@QueryParam("id") String id,
			@DefaultValue("") @QueryParam("nick") String nick,
			@DefaultValue("") @QueryParam("ip") String ip,
			// Seems like JAX-RS/Jersey has problems dealing with non-String data in HTTP-methods
			//@DefaultValue("") @QueryParam("lease") Long lease,
			@DefaultValue("") @QueryParam("lease") String lease,
			@Context HttpServletRequest request) {
		if(nick.equals(""))
			nick = null;
		if(ip.equals(""))
			ip = request.getRemoteAddr();
		// Check value of lease and parse to long
		long leaseLong;
		if(lease.equals("")) {
			leaseLong = this.leaseTime;
		} else {
			leaseLong = System.currentTimeMillis()/1000+Long.parseLong(lease);
		}
		
		//if(lease == 0) // ???
			//lease = System.currentTimeMillis()/1000+this.leaseTime;
		//else
			//lease = System.currentTimeMillis()/1000+lease;
		// Add new peer object
		
		Peer peer = new Peer(id, nick, ip,leaseLong);
		peers.put(id, peer);
		
		return new Response("Peer created.");
		//return "Peer created."; // TODO change to XML-output
	}
	
	// Used by various methods to generate a XML-based result with specific result message
	@XmlRootElement
	static class Response {
		public String result;
		public Response(String result) {
			this.result = result;
		}
		public Response(){} // Seems like JAXB always wants default no-arg construtor, even when no marshalling (Java->XML) is used
	}
	
	
	
}
