package mygroup.myartifact.modules.global.supernode.webservice.test;

import java.util.ArrayList;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.xml.bind.annotation.XmlRootElement;

import mygroup.myartifact.modules.global.supernode.webservice.ListBean;
import mygroup.myartifact.modules.global.supernode.webservice.Peer;


/**
 * Web service to test JAXB feature within JAX-RS.
 * @author daniel
 */

@Path("jaxb")
//@Produces("application/xml") // needed?
public class JAXBWebService {
	
	// Does JAX-RS initializes each WebService with the default constructor? YES
	// For *each* request this constructor gets called and a new object is created to handle the request
	public JAXBWebService() {
		System.out.println("########## ########## ########## ########## ");
		System.out.println("Constructor called ...");
		System.out.println("########## ########## ########## ########## ");
	}
	
	@GET
	//@Consumes(...) // What does this method consumes? xml-formurl...data ???
	//@Produces({"application/json", "application/xml"}) // interesting, {...} and "..." at outer level
	@Produces("application/xml")
	  // Content negotiation used to determine returned serialization format
	// Test this method with curl; you need to set an accept format to resolve the negotiation
	// curl -v -H "Accept: application/xml" http://localhost:8282/rest/jaxb/
	// changing the format to application/json doesn't work, why? not important for now!
	// Look at about:config of Firefox and search 'Accept': Firefox prefers xml, so it gets xml
	public GreetingBean get() { // this time we return a JAXB serialized object
		return new GreetingBean("Hi everyone!"); // a JAXB object, which gets serialized to XML or JSON
		//return new GreetingBean();
	}
	
	@GET
	@Path("test2")
	public ListBean getPeer() {
		Peer peer1 = new Peer("myid", "jack", "1.2.3.4",  100);
		Peer peer2 = new Peer("myid", "joe",  "5.6.7.8",  200);
		Peer peer3 = new Peer("myid", "nick", "11.2.5.6", 300);
		
		List<Peer> list = new ArrayList<Peer>(); // copy of references or wrapper-approach? don't know, hope it works
		list.add(peer1);
		list.add(peer2);
		list.add(peer3);
		
		ListBean listBean = new ListBean(list);
		//return listBean;
		return listBean;
	}
	
	@GET
	@Path("test")
	@Produces("application/xml")
	public Peer getTest() {
		//return new TestBean();
		return new Peer("myid", "jack", "1.2.3.4",  100);
	}
	//class TestBean { // non-static inner classes can't be handled by JAXB
	@XmlRootElement
	static class TestBean {
		public List<Integer> number = new ArrayList<Integer>();
		public TestBean() {
			number.add(1);
			number.add(3);
			number.add(5);
		}
	}
	
	@PUT // not POST=create, gut PUT=update
	//@Consumes({"application/json", "application/xml"})
	@Consumes("application/xml")
	@Produces("application/xml")
	// Test with:
	// curl -v -X PUT --data-binary '<greetingBean><extra>extra text</extra><greetingNode>I say hello!</greetingNode></greetingBean>' -H 'Content-Type: application/xml' -H 'Accept: application/xml' http://localhost:8282/rest/jaxb
	// curl -v -X PUT --data-binary '<greetingBean><greetingNode>I say hello!</greetingNode><extra>extra text</extra></greetingBean>' -H 'Content-Type: application/xml' -H 'Accept: application/xml' http://localhost:8282/rest/jaxb
	// Very interesting: the order of nodes is *not* important for JAXB in the default-case.
	// Seems like the default doesn't use a sequence (fixed-order) but another XML-construct
	public GreetingBean post( GreetingBean bean) {
		// TODO where to save the bean here?
		return bean; // Awesome, it works :)
	}
	
	// TODO make a GET method to list all stored beans in memory
	//      also thing about a lease-time to through away old entries
	//      this needs some sort of cronjob, better at-jpb, to be executed when a lease time ends 
	
	
}
