package mygroup.myartifact.modules.global.supernode.webservice.test;

import java.math.BigDecimal;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

import mygroup.myartifact.modules.global.supernode.webservice.ListBean;

import com.sun.jersey.spi.resource.Singleton;

/**
 * Web service to help peers find each other. Peers can register and search for other peers on this web service. Each
 * registration has a lease time associated with. A registration is thrown away by the web service, when the registered
 * peer doesn't renewed the registration and the lease time expired.
 * 
 * 
 * Peers can do the following on this web service:
 * -Register
 *  A peer can register itself on the web service with the POST-method /register. The peer supplies a unique identifier
 *  and optionally a nickname.
 *  (Which IP to store? Should the peer supply it's (maybe) local IP? Or should the web service use the publicly
 *  available IP?)
 * -Renew (is this really necessary? Why not use a Register again?)
 * -Deregister
 *  A peer can deregister itself, e.g. when the peer is shutting down or it doesn't want to be found by other peers
 *  anymore. It is a good behavior of a peer to deregister itself before leaving a P2P network, even though it is not
 *  necessary, because the lease time will guarantee that every unavailable peer will be deregistered automatically
 *  after a short period of time.
 * -List
 *  A peer can list all known peers by this web service. The asking peer doesn't need to be registered to get an
 *  up-to-date list of known peers.
 * -Peer
 * 
 */

@Path("numbers") // NOTE without '/' prefix at beginning!
@Singleton // Jersey-specific Annotation
public class NumbersWebService {
	//List<PeerBean> peers = Collections.synchronizedList(new ArrayList<PeerBean>()); // TODO allow duplicates? Bad!
	List<Integer> numbers = Collections.synchronizedList(new ArrayList<Integer>());
	
	public NumbersWebService() throws UnknownHostException {
		System.out.println("NumbersWebService: WebService is a singleton");
		System.out.println("NumbersWebService: Init with dummy data");
		numbers.add(1);
		numbers.add(2);
		numbers.add(3);
	}
	
	@GET
	@Path("list")
	@Produces("application/xml")
	// curl -v http://localhost:8282/rest/numbers/list
	public ListBean<Integer> list() {
		// write XML document with content; use JAXB for easier handling of data
		ListBean<Integer> list = new ListBean<Integer>(numbers);
		return list; // why is a no-arg default constructor needed? the object is already created! I guess I have to understand JAXB first.
		
		// nice, works :)
		// TODO generate URI links to all resources
	}
	
	@GET
	@Path("getAdd")
	//@Consumes("")
	@Produces("text/plain")
	// curl -v -d "n=7" http://localhost:8282/rest/numbers/add
	// curl -v -X POST -d "n=7" http://localhost:8282/rest/numbers/add
	// More RESTful(???): curl -X POST -d @filename http://example.com/path/to/resource
	public String getAdd(@QueryParam("n") BigDecimal n) { // @DefaultValue("")
		System.out.println("n is: "+n);
		//numbers.add(n.intValue());
		return "Added";
	}
	
	@Path("add")
	@POST
	//@Consumes("")
	@Produces("text/plain")
	// curl -v -d "n=7" http://localhost:8282/rest/numbers/add
	// curl -v -X POST -d "n=7" http://localhost:8282/rest/numbers/add
	// More RESTful(???): curl -X POST -d @filename http://example.com/path/to/resource
	public String add(@FormParam("n") BigDecimal n) {
		numbers.add(n.intValue());
		return "Added";
	}
	
	@Path("delete/{id}")
	@DELETE
	@Produces("text/plain")
	//      curl -v -X DELETE http://localhost:8282/rest/numbers/delete/{id}
	// e.g. curl -v -X DELETE http://localhost:8282/rest/numbers/delete/7
	public String delete(@PathParam("id") BigDecimal n) {
		//numbers.add(n.intValue());
		//numbers.remove(n);
		numbers.remove(0);
		return "Removed";
	}
	
}
