package mygroup.myartifact.modules.messages;


/**
 * The interface NodeMessage introduces a hierarchy of message types to enable type-safe communication between
 * different nodes on node-level.
 * 
 * The hierarchy of messages types looks like this:
 * 
 * NodeMessage
 * -getSender():String
 * |
 * |---BroadcastMessage
 * |   |
 * |   |---BroadcastDispatchers
 * |   |   -getDispatchers():List<String>
 * |   |
 * |   |---BroadcastNickname
 * |   |   -getNick():String
 * |   |
 * |   |---BroadcastNodeInfo
 * |       -getNodeInfo():NodeInfo
 * |       
 * |---UnicastMessage
 *     -getReceiver():String
 *     |---TellNodeInfo
 *         -getNodeInfo():NodeInfo
 * 
 * 
 * To spare concrete classes from implementing common features again, there are some abstract classes which implement
 * interfaces and provide a simple implementation for needed information:
 *  - NodeMessageDefault    implements NodeMessage    with getSender():String
 *  - UnicastMessageDefault implements UnicastMessage with getReceiver():String
 * 
 * 
 * @author dell
 *
 */
public interface NodeMessage {
	public String getSender();
	//public boolean isType(Class<?> cls); // Use this method to check if a dispatcher can understand this message or not
	//public String getType(); // Use only for debugging purpose when a Dispatcher cannot understand a message; or integrate type into toString() method?
	
	// TODO why not introduce another marker interfaces, e.g. ChatMessage to distinguish different message-types and be able to forward them to right dispatcher
	//      this would make type-property obsolete, which is good, because it is confusing anyway
	//      maybe there will be a good case for using type in the near future, but no currently there is no need for this
	//      This makes also the Class parameter obsolete
	
	// Wieso nicht auch jeder NodeMessage eine eindeutige message-id geben? Macht sicherlich vieles sehr viel leichter
	public String getMessageId();
}
