package mygroup.myartifact.modules.multicast;

import java.net.InetSocketAddress;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import mygroup.myartifact.modules.Node;

/**
 * This class uses a Node reference and supply it with additional MulticastModule-specific information, most notably
 * with IP and Port information (InetSocketAddress) of found node.
 * 
 * @author dell
 *
 */
public class MulticastNode {
	private static final int DEFAULT_LEASE = 120;
	
	private Logger _logger;
	private String _id;
	
	private InetSocketAddress _address;
	private int _leaseTime;
	
	public MulticastNode(String id, InetSocketAddress address) {
		_logger = LoggerFactory.getLogger(MulticastNode.class);	
		_id = id;
		_address = address;
		_leaseTime = DEFAULT_LEASE;
	}
	
	public InetSocketAddress getAddress() {
		return _address;
	}
	
	public String getId() {
		return _id;
	}
	
	public void resetLease() {
		_leaseTime = DEFAULT_LEASE;
	}
	
	public void decreaseLease(int decrease) {
		if(_leaseTime - decrease < 0) {
			_leaseTime = 0;			
		} else {
			_leaseTime = _leaseTime - decrease;
		}
	}
	
	public boolean isAvailable() {
		return (_leaseTime==0) ? false : true;
	}
}
