package mygroup.myartifact.modules.multicast.messages;

import java.net.InetSocketAddress;

/**
 * Interface to mark a message as a MulticastMessage and use it for inter-module communication.
 * 
 * 
 * TODO is it a good idea to inherit from NodeMessage? It is good to have a sender attribute at hand, but why not introduce itself?
 *      BUT NodeMessage uses String-id as identifier, MulticastMessage only can use IP-Port address!
 *      SO let's introduce sender attribute, but not as String, but as IP-port (=InetSocketAddress).
 *      
 * TODO little problem with setting sender of MulticastMessage: the sender IP and Port and *not* known while creating a
 *      MulticastMessage on the sender side, but not before the message arrives on the receiver side; and there could
 *      be multiple receivers. The sender information is encapsulated in a Datagram Packet? I need to run some tests,
 *      but how to solve this problem?
 *      I think currently the best solution would be to introduce a method setSender(InetSocketAddress) on
 *      MulticastMessage and use this to inject the ip and port data after the receiver received a MulticastMessage, of
 *      course only *after* deserialization and after checking (instanceof) that the received message is really a
 *      MulticastMessage object. This injecting eases handling of received packet, because with this we have a again a
 *      nice object with encapsulated data to use.
 * 
 * @author dell
 *
 */
public interface MulticastMessage {
	// TODO want to switch from InetSocketAddress(IP+Port) to InetAddress(IP)? Possible alternative
	public InetSocketAddress getSender();
	public void setSender(InetSocketAddress sender);
	
	// Need to introduce node-ID on MulticastMessage-level as well to enable case where *two* MulticastModules within their own MachineNode are running on the *same* host/machine
	// What about multiple MulticastModules running within the same MachineNode? Well, this is useless and MulticastModule should be grumpy when detecting another MulticastModule already running and refuse to start/work
	public String getSenderNodeId();
	
	// Mir reicht es, ab sofort möchte ich, dass *jede* MulticastMessage eine eindeutige ID vom Sender erhält
	// Die ID besteht aus Unix-Timestamp und der ID des knoten in Kombination: <unix-time>-<node-id>
	// Das sollte ausreichend sein; so sollte es auch einfach sein einen Request-Response-Mechanismus einzuführen
	// Wer soll die message-id setzen? Der Protocolhandler beim Senden einer Multicastnachricht bzw. wenn eine MulticastNachricht erzeugt wird
	// Oder am besten intern im Konstruktur die ID erzeugen, einfach aktuelle Unix-Timestamp nehmen und sender-parameter hinzulegen als node-id
	public String getMessageId();
	
}
