package mygroup.myartifact.modules.multicast.messages;

import java.net.InetSocketAddress;

/**
 * This abstract class implements basic functionality needed for a MulticastMessage to avoid implementing the same
 * logic in every concrete message-type.
 * 
 * @author dell
 *
 */
public abstract class MulticastMessageDefault implements MulticastMessage {
	protected InetSocketAddress _sender;
	protected String _senderNodeId;
	protected String _messageId;
	
	public MulticastMessageDefault(String senderNodeId) {
		_senderNodeId = senderNodeId;
		_messageId = System.currentTimeMillis() + "-" + senderNodeId; // STOP XML Bean Persistence also needs setMessageId() method; I HATE XML Bean Persistence
	}
	
	@Override
	public String getMessageId() {
		return _messageId;
	}
	
	@Override
	public InetSocketAddress getSender() {
		// How to ensure that receiver side set _sender? I guess there is no way to explicitly force a call on setSender();
		// On the other side null is returned if setSender wasn't called, so null can be an indicator for this.
		// Setting on constructor is no solution, because setting sender takes place at receiver-side
		return _sender;
	}

	@Override
	public void setSender(InetSocketAddress sender) {
		_sender = sender;
	}
	
	@Override
	public String getSenderNodeId() {
		return _senderNodeId;
	}
	
	
	// I know, I know: Needed for XML Bean Persistence; use XStream to avoid this ugly handling
	public MulticastMessageDefault() {}
	public void setSenderNodeId(String senderNodeId) {
		_senderNodeId = senderNodeId;
	}
	public void setMessageId(String messageId) {
		_messageId = messageId;
	}
}
