package mygroup.myartifact.modules.multicast.messages;

/**
 * This message-type is used by a local MulticastModule to tell a remote MulticastModule to try to connect with a
 * socket to the port told by this local MultocastModule within this message.
 * 
 * This is (currently) simple one-shot message, because in the context of Multicast it is currently assumed that when
 * a direct connection from A->B is possible, then this also always true for B->A. So with this assumption it is
 * effectively enough to check if a connection A->B is possible.
 * Of course this assumption can be questioned, for the moment assuming this makes interaction between two
 * MulticastModules a lot easier.
 * 
 * OK here I can clearly see that also each MulticastMessage needs to carry node-ID as well to enable unique handling of MulticastMessages
 *    I simply forgot to handle the case where *two* MulticastModules run on the machine in *two* different MachineNodes
 *    let MulticastModules carry node-ID should be able to handle to solve this kind of problem
 *    
 *    this also introduces another case: a MulticastModule wants to be the only MultocastModule within a single MachineNode or both of them will receive the same message
 *    Two possible solutions to this problem: 1. give each MulticastModule an unique name as well
 *                                               TODO sound like a very interesting approach, need to think about this further for another time
 *                                            2. make MulticastModule, or MachineNode at all, grumpy when it detects another MulticastModule as already available and refusing to work
 *    Which solution to apply? let's make MulticastModule grumpy :) is also the most easiest solution to apply
 * 
 * @author dell
 * 
 */
public class RequestConnecting extends MulticastMessageDefault {
	private int _port;
	private String _dispatcherClass;
	
	// Most MulticastMessages are send to all nodes (well, that's why it is called a MulticastMessage); but some MulticastMessages should only be processed by a single node.
	// And that's the case for RequestConnectiong where a node A tells a *single* other node B to connect to a specific port and handle over connection-handling to a specific dispatcher.
	// SO use current policy: MulticastMessage has no receiver; but in cases where receiver is needed, introduce it for specific message-types
	private String _receiverNodeId;

	public RequestConnecting(String senderNodeId, String receiverNodeId, int port, String dispatcherClass) { // Remember: _sender is set *after* receiving a message on destination with MulticastMessage.setSender(InetSocketAddress)
		super(senderNodeId);
		_port = port;
		_dispatcherClass = dispatcherClass;
		_receiverNodeId = receiverNodeId;
	}

	public int getPort() {
		return _port;
	}
	
	public String getDispatcherClass() {
		return _dispatcherClass;
	}
	
	public String getReceiverNodeId() {
		return _receiverNodeId;
	}

	// Unfortunately needed for XML Bean Persistence
	public RequestConnecting() {}
	public void setPort(int port) { _port = port; }
	public void setDispatcherClass(String dispatcherClass) { _dispatcherClass = dispatcherClass; }
	public void setReceiverNodeId(String receiverNodeId) { _receiverNodeId = receiverNodeId; }
}
