package mygroup.myartifact.modules.websocket;

import java.net.InetAddress;

import mygroup.myartifact.modules.Node;

/**
 * Each module can store module-specific data about node and deal with it in module-specific protocol. A WebsocketModule
 * uses WebsocketNodes to store such information.
 * 
 * A WebsocketNode is *not* a subclass if Node, but a wrapper, holding a reference to a Node instance. A single node
 * could be found by different modules and each module introduces own information.
 * 
 * A WebsocketServer sees the public IP of a node. But this should be used careful, because both, node and
 * WebsocketServer, could be in a separate local network or even run on the same local machine. In this case the
 * 'public' IP is only 'public' in the perspective of the WebsocketServer but not in the perspective of other remote
 * nodes.
 * 
 * Currently not used, because no extra information needed.
 * 
 * @author dell
 */
public class WebsocketNode {
	private Node node;
	
	// Currently there is no WebSocket-specific data, but this maybe changes in the near future
	
	// Publicly visible (from server view) IP address of remote node; is this robust? No. So how to get IP of remote node?
	// But OK if strategy to establish direct connection is tested from both sides: A tries to connect to B, and B tries to connect to A.
	// This way an IP determined this way is robust again, but only in this case.
	private InetAddress _ip; 
	
	WebsocketNode(Node node) {
		this.node = node;
	}
	
	public Node getNode() {
		return node;
	}
	
	// How to compare two WebSocket instance with each other? Maybe based on contained node reference? Or based on ID of contained node? 
	// Equals-compare based on inner-node instance OK, because Node.equals(Node) effectively also checks based on id of node
	
}
