package mygroup.myartifact.modules.websocket.messages;

/**
 * A node A tells WebsocketServer to establish a relay connection to node B. Node A uses this way to create a socket
 * connection to node B as last fallback when creating a direct connection failed, e.g. due to present NAT-firewalls
 * on both sides.
 * 
 * A WebsocketServer receiving this message opens two ports X and Y, tell node A about X, tells B about Y and waits for
 * both to connect, which should be possible without any problems.
 * 
 * This message-type is part of a request-response-cycle between node A and WebsocketServer.
 * 
 * @author dell
 *
 */
public class EstablishProxyConnection extends WebsocketMessage {
	private String _destinationNodeId; // ID of node to which a relay connection should be established
	private String _dispatcherClass; // Some dispatcher needs to handle direct connection
	
	public EstablishProxyConnection(String destinationNodeId, String dispatcherClass) {
		_destinationNodeId = destinationNodeId;
		_dispatcherClass = dispatcherClass;
	}
	
	public String getDestinationNodeId() {
		return _destinationNodeId;
	}
	
	public String getDispatcherClass() {
		return _dispatcherClass;
	}
	
	
	// Needed for XML Bean Persistence
	public EstablishProxyConnection() {}
	public void setDestinationNodeId(String destinationNodeId) { _destinationNodeId = destinationNodeId; }
	public void setDispatcherClass(String dispatcherClass) { _dispatcherClass = dispatcherClass; }
}
