package mygroup.myartifact.modules.websocket.server;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.CountDownLatch;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HandleClientConnection implements Runnable {
	private Logger _logger;
	private HandleClientConnection _handler;
	private ServerSocket _server;
	private Socket _otherSocket;
	private CountDownLatch _latch;
	
	public HandleClientConnection(ServerSocket server, CountDownLatch latch) {
		_logger = LoggerFactory.getLogger(HandleClientConnection.class);
		_server = server;
		_latch = latch;
	}
	
	public void setHandler(HandleClientConnection handler) {
		_handler = handler;
	}
	
	public void setSocket(Socket otherSocket) {
		_logger.debug("Setting otherSocket...");
		_otherSocket = otherSocket;
		_logger.debug("Done settign otherSocket.");
	}
	
	@Override
	public void run() {
		try {
			Socket client = _server.accept();
			_logger.debug("Node connected on local port {}, waiting for other node to connect...", client.getLocalPort());
			_handler.setSocket(client); // WHAT? What is going wrong here!?!
			_latch.countDown();
			
			// Wire connection with other connection after it is sure that the other connection is established as well and reference to Socket was set from other Handler
			try {
				_latch.await();
				_logger.debug("Both nodes connected, wiring connections...");
				
				InputStream in = client.getInputStream();
				OutputStream otherOut = _otherSocket.getOutputStream();
				
				_logger.debug("Got InputStream on port {} and OutputStream on port {}.", new Object[]{client.getLocalPort(), _otherSocket.getLocalPort()});
				
				byte[] buffer = new byte[1024*8]; // 8KB
				int bytesRead = 0;
				while( (bytesRead=in.read(buffer))>0 ) {
					_logger.debug("Read {} bytes.", bytesRead);
					otherOut.write(buffer, 0, bytesRead);
					_logger.debug("Wrote {} bytes.", bytesRead);
				}
				_logger.debug("Closing connection on local port {} and informing other connection about close.", client.getLocalPort());
				client.close();
				_otherSocket.close(); // Tell other client as well that connection on this connection ended
			} catch (InterruptedException e) {
				_logger.debug("Waiting on CountDownLatch interrupted, this should never happen, program in undefined state.");
				e.printStackTrace();
			}
		} catch (IOException e) {
			_logger.debug("Client could not connect to opened port, aborting establishing relay connection.");
		}
	}

}
